<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Class Auth
 * @property Auth_model auth_model
 */
class Auth extends MY_Controller
{

    function __construct()
    {
        parent::__construct(true, false);

        // Load required CI libraries and helpers.
        $this->load->helper('form');

        // Redirect users logged in via password (However, not 'Remember me' users, as they may wish to login properly).
        if ($this->flexi_auth->is_logged_in_via_password() && $this->flexi_auth->is_admin()) {
            // Preserve any flashdata messages so they are passed to the redirect page.
            if ($this->session->flashdata('message')) {
                $this->session->keep_flashdata('message');
            }

            // Redirect logged in admins (For security, admin users should always sign in via Password rather than 'Remember me'.
            redirect('admin');
        }

        // Define a global variable to store data that is then used by the end view page.
        $this->data = null;
    }

    public function index()
    {
        $this->login();
    }

    function login()
    {
        // If 'Login' form has been submited, attempt to log the user in.
        if ($this->input->post('login_user')) {
            $this->load->model('auth_model');
            $this->auth_model->adminLogin();
        }

        // CAPTCHA Example
        // Check whether there are any existing failed login attempts from the users ip address and whether those attempts have exceeded the defined threshold limit.
        // If the user has exceeded the limit, generate a 'CAPTCHA' that the user must additionally complete when next attempting to login.
        if ($this->flexi_auth->ip_login_attempts_exceeded()) {
            /**
             * reCAPTCHA
             * http://www.google.com/recaptcha
             * To activate reCAPTCHA, ensure the 'recaptcha()' function below is uncommented and then comment out the 'math_captcha()' function further below.
             *
             * A boolean variable can be passed to 'recaptcha()' to set whether to use SSL or not.
             * When displaying the captcha in a view, reCAPTCHA generates all html required for display.
             *
             * Note: To use this example, you will also need to enable the recaptcha examples in 'models/demo_auth_model.php', and 'views/demo/login_view.php'.
             */
            $this->data['captcha'] = $this->flexi_auth->recaptcha(FALSE);

            /**
             * flexi auths math CAPTCHA
             * Math CAPTCHA is a basic CAPTCHA style feature that asks users a basic maths based question to validate they are indeed not a bot.
             * For flexibility on CAPTCHA presentation, the 'math_captcha()' function only generates a string of the equation, see the example below.
             *
             * To activate math_captcha, ensure the 'math_captcha()' function below is uncommented and then comment out the 'recaptcha()' function above.
             *
             * Note: To use this example, you will also need to enable the math_captcha examples in 'models/demo_auth_model.php', and 'views/demo/login_view.php'.
             */
            # $this->data['captcha'] = $this->flexi_auth->math_captcha(FALSE);
        }

        // Get any status message that may have been set.
        $this->data['message'] = (!isset($this->data['message'])) ? $this->session->flashdata('message') : $this->data['message'];

        $this->admin_master_view('admin/login', $this->data);
    }

    /**
     * login_via_ajax
     * A simplified version of the above 'login' method that instead uses ajax to submit a users credentials.
     * This demo includes 3 example accounts that can be logged into via using either their email address or username. The login details are provided within the view page.
     * Note: This page is only accessible to users who are not currently logged in, else they will be redirected.
     */
    function login_via_ajax()
    {
        if ($this->input->is_ajax_request()) {
            $this->load->model('auth_model');

            $this->auth_model->login_via_ajax();

            die($this->flexi_auth->is_logged_in());
        } else {
            $this->load->view('user/login_via_ajax_view', $this->data);
        }
    }

}