<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Class Main
 * @property Auth_admin_model auth_admin_model
 */
class Main extends MY_Controller
{
    function __construct()
    {
        parent::__construct(true);

        // Load required CI libraries and helpers.
        $this->load->helper('form');
        $this->load->model('ticket_model');

        // Define a global variable to store data that is then used by the end view page.
        $this->data = null;

        $this->data['types'] = $this->ticket_model->get_types(true);
    }

    public function index()
    {
        $this->admin_master_view('admin/main', $this->data);
    }

    /**
     * manage_user_accounts
     * View and manage a table of all users.
     * This example allows accounts to be suspended or deleted via checkoxes within the page.
     * The example also includes a search tool to lookup users via either their email, first name or last name.
     */
    public function manage_user_accounts()
    {
        $this->load->library('flexi_auth');

        //povolit pristup iba pre admina, skupina 2
        if (!$this->flexi_auth->in_group(2)) {
            $this->session->set_flashdata('error', 'Nemáte práva pre prístup do tejto sekcie.');
            redirect('admin');
        }

        $this->load->model('auth_admin_model');

        // If 'Admin Search User' form has been submitted, this example will lookup the users email address and first and last name.
        if ($this->input->post('search_users') && $this->input->post('search_query')) {
            // Convert uri ' ' to '-' spacing to prevent '20%'.
            // Note: Native php functions like urlencode() could be used, but by default, CodeIgniter disallows '+' characters.
            $search_query = str_replace(' ', '-', $this->input->post('search_query'));

            // Assign search to query string.
            redirect('admin/manage_user_accounts/search/' . $search_query . '/page/');
        } // If 'Manage User Accounts' form has been submitted and user has privileges to update user accounts, then update the account details.
        else if ($this->input->post('update_users')) {
            $this->auth_admin_model->update_user_accounts();
        }

        // Get user account data for all users.
        // If a search has been performed, then filter the returned users.
        $this->auth_admin_model->get_user_accounts();

        // Set any returned status/error messages.
        $this->data['message'] = (!isset($this->data['message'])) ? $this->session->flashdata('message') : $this->data['message'];

        $this->admin_master_view('admin/manage_user_acc', $this->data);
    }

    /**
     * update_user_account
     * Update the account details of a specific user.
     */
    function update_user_account($user_id)
    {

        //povolit pristup iba pre admina, skupina 2
        if (!$this->flexi_auth->in_group(2)) {
            $this->session->set_flashdata('error', 'Nemáte práva pre prístup do tejto sekcie.');
            redirect('admin');
        }

        if (empty($user_id)) {
            redirect('admin/manage_user_accounts');
        }

        // If 'Update User Account' form has been submitted, update the users account details.
        if ($this->input->post('update_users_account')) {
            $this->load->model('auth_admin_model');
            $this->auth_admin_model->update_user_account($user_id);
        }

        // Get users current data.
        $sql_where = array($this->flexi_auth->db_column('user_acc', 'id') => $user_id);
        $this->data['user'] = $this->flexi_auth->get_users_row_array(FALSE, $sql_where);

        if (empty($this->data['user'])) {
            $this->session->set_flashdata('message', 'Non-existing user');
            redirect('admin/manage_user_accounts');
        }

        // Get user groups.
        $this->data['groups'] = $this->flexi_auth->get_groups_array();

        // Set any returned status/error messages.
        $this->data['message'] = (!isset($this->data['message'])) ? $this->session->flashdata('message') : $this->data['message'];

        $this->admin_master_view('admin/user_account_update', $this->data);
    }

    function list_lotteries()
    {
        $this->data['title'] = "Choose lottery type";

        $this->data['types'] = $this->ticket_model->get_types(true);

        $this->admin_master_view('admin/choose-type', $this->data);
    }

    public function add_result($id = NULL)
    {

        if ($this->input->post('save_result')) {
            if ($this->input->post('type')) {
                $fname = 'save' . $this->input->post('type');
                if (method_exists($this, $fname)) {
                    $this->$fname();
                } else {
                    $this->data['message'] = 'Error saving your lottery result';
                }
            } else {
                $this->data['message'] = 'Error saving lottery result';
            }
        }

        if (empty($id)) {
            $this->index();
        } else {

            /*$this->load->library("pagination");
            $this->load->model('ticket_model');

            $config = array();
            $config["base_url"] = site_url() . "/admin/add_result/" . $id . "/page/";
            $config["total_rows"] = $this->ticket_model->get_results($id, true);
            $config["per_page"] = 10;
            $config['use_page_numbers'] = TRUE;
            $config["uri_segment"] = 5;

            $this->pagination->initialize($config);

            $page = ($this->uri->segment(5)) ? $this->uri->segment(5) : 1;
            if (!is_int((int)$page) || !((int)$page > 0)) {
                redirect('admin/add_result');
            }

            $this->data["links"] = $this->pagination->create_links();*/

            $this->data['type'] = $this->ticket_model->get_type($id);
//            $this->data['results'] = $this->ticket_model->get_results($id, false, $config['per_page'], ($page - 1) * $config['per_page']);

            if (empty($this->data['type'])) {
                show_error('Unknown lottery type');
            }

            $this->data['title'] = $this->data['type']['name'];

            $this->admin_master_view('admin/lottery-types/' . $this->data['type']['template'], $this->data);
        }
    }

    public function results_ajax()
    {
        if ($this->input->get('type') && is_numeric($this->input->get('type'))) {
            $type = (int)$this->input->get('type');
            $this->load->model('ticket_model');
            $this->ticket_model->get_results_ajax($type);
        }
    }

    //view lottery result
    public function view_result($id = NULL)
    {
        if (empty($id)) {
            redirect('admin');
        } else {
            $this->data['view'] = true;
            $this->data['result_data'] = $this->ticket_model->get_result($id);
            $this->add_result($this->data['result_data']['type_id']);
        }
    }

    //edit lottery reult
    public function edit_result($id = NULL)
    {
        if ($this->input->post('edit_result')) {
            if ($this->input->post('type')) {
                $fname = 'save' . $this->input->post('type');
                if (method_exists($this, $fname)) {
                    $this->$fname($id);
                } else {
                    $this->data['message'] = 'Error saving your lottery result';
                }
            } else {
                $this->data['message'] = 'Error saving lottery result';
            }
        }

        if (empty($id)) {
            redirect('admin');
        } else {
            $this->data['edit'] = true;
            $this->data['result_data'] = $this->ticket_model->get_result($id);
            $this->add_result($this->data['result_data']['type_id']);
        }
    }

    //delete result
    public function delete_result($id = NULL)
    {
        if (empty($id)) {
            redirect('admin');
        } else {
            $this->ticket_model->delete_result($id);

//            $this->session->set_flashdata('message', 'Result succesfully deleted');
            $this->session->set_flashdata('info', 'Výsledok lotérie úspešne odstránený.');
            redirect($this->input->get('back'));
        }
    }

    //delete result via ajax
    public function delete_result_ajax($id = NULL)
    {
        if (!empty($id)) {
            $this->ticket_model->delete_result($id);
            echo '1';
        }
    }

    //overenie vyssledkov a rozposlanie emailov
    public function check_result($type_id = null)
    {
        if (empty($type_id)) {
            redirect('admin');
        } else {
            $fname = 'check' . $type_id;
            if (method_exists($this, $fname)) {
                $this->$fname();
            } else {
                $this->data['message'] = 'Error checking lottery results';
            }
        }

        redirect('admin/add_result/' . $type_id);
//        $this->add_result($type_id);

    }

    //save ticket result type 1
    private function save1($edit = 0)
    {
        $type = 1;

        $this->form_validation->set_rules('tah1', 'Ťah 1', 'required|tips_count[7]');
        $this->form_validation->set_rules('tah2', 'Ťah 2', 'required|tips_count[7]');
        $this->form_validation->set_rules('sance', 'Šanca', 'required|exact_length_extra[6]');
        $this->form_validation->set_rules('date', 'Dátum losovania', 'required|check_date[' . $type . ';true]');
//        $this->form_validation->set_rules('mimoradna', '', '');
        $this->form_validation->set_rules('info', '', '');

        if ($this->form_validation->run() == TRUE) {
            $res = $this->ticket_model->save_result($type, $edit);

            if (!$res) {
                $this->data['error'] = 'Chyba pri ukladaní výsledku.';
            } else {
                $this->session->set_flashdata('info', 'Výsledok lotérie úspešne uložený.');
                redirect('admin/add_result/' . $type);
                $this->index();
            }
        }
        return true;
    }

    //save ticket result type 2
    private function save2($edit = 0)
    {
        $type = 2;

        $this->form_validation->set_rules('tah1', 'Ťah 1', 'required|tips_count[6]');
        $this->form_validation->set_rules('color[]', 'Farba', 'required|integer');
        $this->form_validation->set_rules('sance', 'Šťastné číslo', 'required|exact_length_extra[7]');
        $this->form_validation->set_rules('date', 'Dátum losovania', 'required|check_date[' . $type . ';true]');
        $this->form_validation->set_rules('info', '', '');

        if ($this->form_validation->run() == TRUE) {
            $res = $this->ticket_model->save_result($type, $edit);

            if (!$res) {
                $this->data['error'] = 'Chyba pri ukladaní výsledku.';
            } else {
                $this->session->set_flashdata('info', 'Výsledok lotérie úspešne uložený.');
                redirect('admin/add_result/' . $type);
                $this->index();
            }
        }
        return true;
    }

    //save ticket result type 3
    private function save3($edit = 0)
    {
        $type = 3;

        $this->form_validation->set_rules('tah1', 'Ťah 1', 'required|tips_count[6]');
        $this->form_validation->set_rules('slos[]', 'Losovanie', 'required|integer');
        $this->form_validation->set_rules('date', 'Dátum losovania', 'required|check_date[' . $type . ';true]');
        $this->form_validation->set_rules('info', '', '');

        if ($this->form_validation->run() == TRUE) {
            $res = $this->ticket_model->save_result($type, $edit);

            if (!$res) {
                $this->data['error'] = 'Chyba pri ukladaní výsledku.';
            } else {
                $this->session->set_flashdata('info', 'Výsledok lotérie úspešne uložený.');
                redirect('admin/add_result/' . $type);
                $this->index();
            }
        }
        return true;
    }

    /*public function startWorker()
    {
        include_once(APPPATH . 'libraries/DJJob.php');
        include_once(APPPATH . 'libraries/Send_delayed_mail.php');

        $hostname = $this->db->hostname;
        $database = $this->db->database;
        $username = $this->db->username;
        $password = $this->db->password;

        DJJob::configure("mysql:host=" . $hostname . ";dbname=" . $database . "", array('mysql_user' => "" . $username . "", 'mysql_pass' => "" . $password . ""));

        $options = array('env' => 'dev', 'queue' => 'email', 'count' => '0', 'sleep' => '10');

        $worker = new DJWorker($options);
        $worker->start();

    }*/

    private function check1()
    {
        $this->load->library('result_verification');

        $type = 1;

        //vyber prvy vysledok losovania z pola
        $tmp = $this->ticket_model->get_results($type, false, 1);
        $lot_result = $tmp[0];

        //vyber vsetky aktivne tikety
        $tickets = $this->ticket_model->get_active_tickets_by_type_and_day($type, $lot_result['day_in_week'], $lot_result['lottery_date']);

        $out_data = array();

        foreach ($tickets as $ticket) {
            $out = array();
            $out['ticket'] = $ticket;
            $out['result'] = $lot_result;

            $from = $ticket['date_from'];
            $when = explode(';', $ticket['what_days']);
            $times = explode(';', $ticket['times']);

            $res_days = $this->result_verification->get_all_ticket_days($from, $when, $times);

            foreach ($res_days as $res_day) {

                $found = false;

                if ($lot_result['lottery_date'] == $res_day) {

                    $found = true;

                    $win = $this->result_verification->check_ticket_result($ticket['ticket_id'], true);

                    //deaktivuj tiket ak sa skontroloval posledny den
                    if ($res_day == $res_days[count($res_days) - 1]) {
                        $this->ticket_model->deactivate_ticket($ticket['ticket_id']);
                    }

                    if (!isset($out_data[$ticket['uacc_id']])) {
                        $out_data[$ticket['uacc_id']] = array();
                    }

                    //zisti ci sa ma poslat email na zaklade email preferences
                    switch ($ticket['mail_prefs']) { //nastavenie emailu pre tiket
                        case 0: //zobrat globalne nastavenie
                            if ($ticket['email_prefs'] == 1) { //1=vzdy
                                array_push($out_data[$ticket['uacc_id']], $out);
                            } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                if ($win)
                                    array_push($out_data[$ticket['uacc_id']], $out);
                            }
                            break;
                        case 1: //vzdy
                            array_push($out_data[$ticket['uacc_id']], $out);
                            break;
                        case 2:
                            //iba pri vyhre
                            if ($win)
                                array_push($out_data[$ticket['uacc_id']], $out);
                            break;
                        default:
                            if ($ticket['email_prefs'] == 1) { //1=vzdy
                                array_push($out_data[$ticket['uacc_id']], $out);
                            } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                if ($win)
                                    array_push($out_data[$ticket['uacc_id']], $out);
                            }
                    }

                    break;
                }

                if (!$found) {
                    //deaktivuj tiket ak uz nie je aktivny
                    $this->check_active($res_days, $ticket['ticket_id']);
                }
            }
        }

        $this->send_mails($out_data);

    }

    private function check2()
    {
        $this->load->library('result_verification');

        $type = 2;

        //vyber prvy vysledok losovania z pola
        $tmp = $this->ticket_model->get_results($type, false, 1);
        $lot_result = $tmp[0];

        //vyber vsetky aktivne tikety
        $tickets = $this->ticket_model->get_active_tickets_by_type_and_day($type, $lot_result['day_in_week'], $lot_result['lottery_date']);

        $out_data = array();

        foreach ($tickets as $ticket) {

            $out = array();
            $out['ticket'] = $ticket;
            $out['result'] = $lot_result;

            $from = $ticket['date_from'];
            $when = explode(';', $ticket['what_days']);
            $times = explode(';', $ticket['times']);

            $res_days = $this->result_verification->get_all_ticket_days($from, $when, $times);

            foreach ($res_days as $res_day) {

                $found = false;

                if ($lot_result['lottery_date'] == $res_day) {

                    $found = true;

                    $win = $this->result_verification->check_ticket_result($ticket['ticket_id'], true);

                    //deaktivuj tiket ak sa skontroloval posledny den
                    if ($res_day == $res_days[count($res_days) - 1]) {
                        $this->ticket_model->deactivate_ticket($ticket['ticket_id']);
                    }

                    if (!isset($out_data[$ticket['uacc_id']])) {
                        $out_data[$ticket['uacc_id']] = array();
                    }

                    //zisti ci sa ma poslat email na zaklade email preferences
                    switch ($ticket['mail_prefs']) { //nastavenie emailu pre tiket
                        case 0: //zobrat globalne nastavenie
                            if ($ticket['email_prefs'] == 1) { //1=vzdy
                                array_push($out_data[$ticket['uacc_id']], $out);
                            } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                if ($win)
                                    array_push($out_data[$ticket['uacc_id']], $out);
                            }
                            break;
                        case 1: //vzdy
                            array_push($out_data[$ticket['uacc_id']], $out);
                            break;
                        case 2:
                            //iba pri vyhre
                            if ($win)
                                array_push($out_data[$ticket['uacc_id']], $out);
                            break;
                        default:
                            if ($ticket['email_prefs'] == 1) { //1=vzdy
                                array_push($out_data[$ticket['uacc_id']], $out);
                            } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                if ($win)
                                    array_push($out_data[$ticket['uacc_id']], $out);
                            }
                    }

                    break;
                }

                if (!$found) {
                    //deaktivuj tiket ak uz nie je aktivny
                    $this->check_active($res_days, $ticket['ticket_id']);
                }
            }
        }

        $this->send_mails($out_data);

    }

    private function check3()
    {
        $this->load->library('result_verification');

        $type = 3;

        //vyber prvy vysledok losovania z pola
        $tmp = $this->ticket_model->get_results($type, false, 1);
        $lot_result = $tmp[0];

        //vyber vsetky aktivne tikety
        $tickets = $this->ticket_model->get_active_tickets_by_type_and_day($type, $lot_result['day_in_week'], $lot_result['lottery_date']);

        $out_data = array();

        foreach ($tickets as $ticket) {

            $additional = json_decode($ticket['additional'], true);
            $slos = $additional['slos'];

            $out = array();
            $out['ticket'] = $ticket;
            $out['result'] = $lot_result;

            $from = $ticket['date_from'];
            $when = explode(';', $ticket['what_days']);
            $times = explode(';', $ticket['times']);

            $res_days = $this->result_verification->get_all_ticket_days($from, $when, $times, $type);

            foreach ($res_days as $res_day) {

                $found = false;

                if ($lot_result['lottery_date'] == $res_day) {

                    $found = true;

                    if (in_array($lot_result['order'], $slos)) {

                        $win = $this->result_verification->check_ticket_result($ticket['ticket_id'], true);

                        //deaktivuj tiket ak sa skontroloval posledny den
                        if ($res_day == $res_days[count($res_days) - 1] && $lot_result['order'] == 2) {
                            $this->ticket_model->deactivate_ticket($ticket['ticket_id']);
                        }

                        if (!isset($out_data[$ticket['uacc_id']])) {
                            $out_data[$ticket['uacc_id']] = array();
                        }

                        //zisti ci sa ma poslat email na zaklade email preferences
                        switch ($ticket['mail_prefs']) { //nastavenie emailu pre tiket
                            case 0: //zobrat globalne nastavenie
                                if ($ticket['email_prefs'] == 1) { //1=vzdy
                                    array_push($out_data[$ticket['uacc_id']], $out);
                                } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                    if ($win)
                                        array_push($out_data[$ticket['uacc_id']], $out);
                                }
                                break;
                            case 1: //vzdy
                                array_push($out_data[$ticket['uacc_id']], $out);
                                break;
                            case 2:
                                //iba pri vyhre
                                if ($win)
                                    array_push($out_data[$ticket['uacc_id']], $out);
                                break;
                            default:
                                if ($ticket['email_prefs'] == 1) { //1=vzdy
                                    array_push($out_data[$ticket['uacc_id']], $out);
                                } else if ($ticket['mail_prefs'] == 2) { //2=iba ak je vyhra
                                    if ($win)
                                        array_push($out_data[$ticket['uacc_id']], $out);
                                }
                        }

                        break;
                    }
                }

                if (!$found) {
                    //deaktivuj tiket ak uz nie je aktivny
                    $this->check_active($res_days, $ticket['ticket_id']);
                }
            }
        }

        $this->send_mails($out_data);

    }

    private function check_active($res_days, $ticket_id)
    {

        $today = strtotime(date("Y-m-d"));
        $last_day = strtotime(end($res_days));

        if ($last_day < $today) {
            $this->ticket_model->deactivate_ticket($ticket_id);
        }
    }

    private function send_mails($out_data)
    {
        $success = 0;
        $error = 0;

        include_once(APPPATH . 'libraries/DJJob.php');
        include_once(APPPATH . 'libraries/Send_delayed_mail.php');

        foreach ($out_data as $user) {

            //ak nie je ziadny tiket na odoslanie, podla emailovy preferencii
            if (isset($user[0])) {
                $data = $user[0];
            } else {
                continue;
            }

            $out['data'] = $user;
            $date = iconv(null, "utf-8", strftime("%A, %d. %B %Y", strtotime($data['result']['lottery_date'])));
            $title = 'Výsledky lotérie ' . $data['ticket']['name'] . ' - ' . $data['ticket']['company'] . ' zo dňa ' . $date . ' [' . $this->config->item('email_title') . ']';

//            $this->result_verification->send_email($data['ticket']['uacc_email'], $title, $out, $data['ticket']['template']);

            $hostname = $this->db->hostname;
            $database = $this->db->database;
            $username = $this->db->username;
            $password = $this->db->password;

            DJJob::configure("mysql:host=" . $hostname . ";dbname=" . $database . "", array('mysql_user' => "" . $username . "", 'mysql_pass' => "" . $password . ""));

            /*for ($i = 0; $i < 4; $i++) {
                if (DJJob::enqueue(new Send_delayed_mail($data['ticket']['uacc_email'], $title, $out, $data['ticket']['template']), "email")) {
                    $success++;
                } else {
                    $error++;
                }
            }*/

            if ($this->result_verification->send_email($data['ticket']['uacc_email'], $title, $out, $data['ticket']['template'])) {
                $success++;
            } else {
                $error++;
            }
        }

        $this->session->set_flashdata('info', '<p>Úspešne odoslaných emailov: ' . $success . '</p><p>Neúspešne odoslaných emailov: ' . $error . '</p>');

//        echo 'Pocet uspesne odoslanych emailov: ' . $success;
//        echo br();
//        echo 'Pocet neuspesne odoslanych emailov: ' . $error;

    }
}