<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Class Account
 * @property Ticket_model ticket_model
 * @property Result_verification result_verification
 */
class Account extends MY_Controller
{

    function __construct()
    {
        parent::__construct(false, true, array('update_email'));

        $this->load->helper('form');

        $this->data = null;
    }

    public function _remap($method, $params = array())
    {
        switch ($method) {
            case lang('link_tickets'):
                return call_user_func_array(array($this, 'tickets'), $params);
                break;
            case lang('link_edit'):
                return call_user_func_array(array($this, 'edit'), $params);
                break;
            case lang('link_update_email'):
                return call_user_func_array(array($this, 'update_email'), $params);
                break;
            case 'tickets_ajax':
                return call_user_func_array(array($this, 'tickets_ajax'), $params);
                break;
            case lang('link_statistics'):
                return call_user_func_array(array($this, 'statistics'), $params);
                break;
            default:
                $this->index();
        }
    }

    public function index()
    {
        $data['title'] = "Váš účet";
        $this->master_view('user/account', $data, "acc");
    }

    public function edit()
    {

        // If 'Update Account' form has been submitted, update the user account details.
        if ($this->input->post('update_account')) {
            $this->load->model('auth_model');
            $this->auth_model->update_account();
        }

        if ($this->input->post('change_password')) {
            $this->load->model('auth_model');
            $this->auth_model->change_password();
        }

        if ($this->input->post('update_prefs')) {
            $this->load->model('auth_model');
            $this->auth_model->change_prefs();
        }

        // Get users current data.
        // This example does so via 'get_user_by_identity()', however, 'get_users()' using any other unqiue identifying column and value could also be used.
        $this->data['user'] = $this->flexi_auth->get_user_by_identity_row_array();

        if (empty($this->data['user'])) {
            $this->flexi_auth->logout(FALSE);
            redirect(lang('link_auth'));
        }

        $this->data['title'] = "Upraviť profil";
        $this->master_view('user/profile_edit', $this->data);
    }

    function update_email($user_id = FALSE, $token = FALSE)
    {
        $this->load->model('auth_model');

        // If 'Update Email' form has been submitted, send a verification email to the submitted email address.
        if ($this->input->post('update_email')) {
            $this->auth_model->send_new_email_activation();
        } // Else if page has been accessed via a link set in the verification email, then validate the activation token and update the email address.
        else if (is_numeric($user_id) && strlen($token) == 40) // 40 characters = Email Activation Token length.
        {
            $this->auth_model->verify_updated_email($user_id, $token);
        }

        // In this demo, the 'update_email' page is the only page in this controller that can be accessed without needing to be logged in.
        // This is because, some users may validate their change of email address at a later time, or from a different device that they are not logged in on.
        // Therefore, check that the user is logged in before granting them access to the 'update_email' page.
        if ($this->flexi_auth->is_logged_in()) {
            $this->edit();
        } else {
            redirect(lang('link_auth'));
        }
    }

    //view all saved tickets
    public function tickets()
    {
        /*$this->load->library("pagination");
        $this->load->model('ticket_model');

        $config = array();
        $config["base_url"] = site_url() . '/' . lang('link_account') . '/' . lang('link_tickets') . '/' . lang('link_page');
        $config["total_rows"] = $this->ticket_model->get_tickets(true);
        $config["per_page"] = 10;
        $config['use_page_numbers'] = TRUE;
        $config["uri_segment"] = 4;

        $this->pagination->initialize($config);

        $page = ($this->uri->segment(4)) ? $this->uri->segment(4) : 1;
        if (!is_int((int)$page) || !((int)$page > 0)) {
            redirect(lang('link_account') . '/' . lang('link_tickets'));
        }

        $this->data['tickets'] = $this->ticket_model->get_tickets(false, $config['per_page'], ($page - 1) * $config['per_page']);

        $this->data["links"] = $this->pagination->create_links();*/

        $this->data['title'] = "Zoznam tiketov";
        $this->master_view('user/list_tickets', $this->data);
    }

    public function tickets_ajax()
    {
        $this->load->model('ticket_model');
        $this->ticket_model->get_tickets_ajax();
    }

    public function statistics()
    {
        $this->load->model('ticket_model');
        $this->load->library('result_verification');

        $data = array();

        $data['tickets'] = $this->ticket_model->get_tickets();
        $data['ticket_types'] = $this->ticket_model->get_types();
        $data['tickets_count'] = 0;
        $data['tickets_count_ballot'] = 0; //celkovy pocet zrebovani vsetkych tiketov
        $data['tickets_types_count'] = array();
        $data['tickets_win'] = 0;
        $data['tickets_loss'] = 0;
        $data['count_tips'] = 0;
        $data['count_intersect'] = 0;
        $data['tips_intersect'] = array();
        $data['tips_all'] = array();
        $data['count_tips_all'] = 0;
        $data['count_cols'] = 0;
        $data['count_cols_all'] = 0;
        $data['winning_cols'] = 0;
        $data['korunka_types_count'] = array();

        foreach ($data['tickets'] as $ticket) {
            //filtrovanie start
            $date1 = false;
            $date2 = false;

            $filter_by_date = false;
            $filter_by_lottery = false;

            if ($this->input->get('lottery') && is_numeric($this->input->get('lottery'))) {
                $filter_by_lottery = true;
                $filter_lottery = htmlspecialchars($this->input->get('lottery'), true);
                $this->data['lottery'] = $this->input->get('lottery', true);
            }

            if ($this->input->get('date_from')) {
                if ($date1 = $this->check_date($this->input->get('date_from', true))) {
                    $this->data['date_from'] = $date1;
                    $filter_by_date = true;
                }
            }

            if ($this->input->get('date_to')) {
                if ($date2 = $this->check_date($this->input->get('date_to', true))) {
                    $this->data['date_to'] = $date2;
                    $filter_by_date = true;
                }
            }

            $from = $ticket['date_from'];
            $when = explode(';', $ticket['what_days']);
            $times = explode(';', $ticket['times']);
            $type_id = $ticket['type_id'];
            $result_days = $this->result_verification->get_all_ticket_days($from, $when, $times, $type_id);

            if ($filter_by_date) {
                if ($date1 && $date2) {
                    if (!(reset($result_days) >= $date1 && end($result_days) <= $date2)) {
                        continue;
                    }
                } else if ($date1) {
                    if (!(reset($result_days) >= $date1)) {
                        continue;
                    }
                } else {
                    if (!(end($result_days) <= $date2)) {
                        continue;
                    }
                }
            }

            if ($filter_by_lottery) {
                if (!($ticket['type_id'] == $filter_lottery)) {
                    continue;
                }
            }
            //filtrovanie end

            $data['tickets_count']++;
            $data['tickets_count_ballot'] += count($result_days);

            $res = $this->result_verification->check_ticket_result($ticket['ticket_id'], false, true);

            if ($res[0]) {
                $data['tickets_win']++;
            } else {
                $data['tickets_loss']++;
            }

            $data['tickets_types_count'][$ticket['type_id']] = !isset($data['tickets_types_count'][$ticket['type_id']]) ? 1 : $data['tickets_types_count'][$ticket['type_id']] + 1;
            $data['count_tips'] += $res[1];
            $data['count_intersect'] += $res[2];
            $data['tips_intersect'] = array_merge($data['tips_intersect'], $res[3]);
//            $data['count_cols'] += $res[4]; //nepouziva sa
            $data['winning_cols'] += $res[5];

            //statistika pocetnosti tipovanych cisel
            $ticket_data = $this->ticket_model->get_ticket($ticket['ticket_id']);
            $tips = json_decode($ticket_data['tip'], true);
            foreach ($tips as $key2 => $tip1) {
                $tip = $tip1['tip'];
                if (empty($tip)) {
                    continue;
                }

                $data['count_cols']++;

                if ($ticket['type_id'] == 3) { //korunka
                    $data['korunka_types_count'][$key2] = !isset($data['korunka_types_count'][$key2]) ? 1 : $data['korunka_types_count'][$key2] + 1;
                }

                //tipy sa nasobia poctom losovani tiketu
                foreach ($result_days as $day) {
                    $data['tips_all'] = array_merge($data['tips_all'], $tip);
                    $data['count_tips_all'] += count($tip);
                    $data['count_cols_all']++;

                    if (isset($tip1['color'][0])) {
                        $data['tips_all'] = array_merge($data['tips_all'], ($tip1['color'][0]) == 1 ? array('biela') : array('čierna'));
                        $data['count_tips_all'] += 1;
                    }
                }
            }
        }

        $data['tips_all'] = array_count_values($data['tips_all']);

        $data['tips_intersect'] = array_count_values($data['tips_intersect']);

        $this->data['data'] = $data;

        $this->data['title'] = "Štatistiky";
        $this->master_view('user/statistics', $this->data, 'statistics');
    }

    private function check_date($date)
    {
        if (strpos($date, '.') !== false) {
            $tmp = explode('.', $date);
            if (count($tmp) == 3) {
                $date = trim($tmp[1]) . '/' . trim($tmp[0]) . '/' . trim($tmp[2]);
            }
        }

        $date = date_parse($date);

        if (checkdate($date["month"], $date["day"], $date["year"])) {
            return date("Y-m-d", mktime(0, 0, 0, $date['month'], $date['day'], $date['year']));
        } else {
            return false;
        }
    }

}