<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * @property Ticket_model $ticket_model
 * @property Result_verification result_verification
 */
class Ticket extends MY_Controller
{

    function __construct()
    {
        parent::__construct();

        $this->load->model('ticket_model');
        $this->load->helper('form');
        $this->data = null;
    }

    public function _remap($method, $params = array())
    {
        switch ($method) {
            case lang('link_view'):
                return call_user_func_array(array($this, 'view'), $params);
                break;
            case lang('link_add'):
                return call_user_func_array(array($this, 'add'), $params);
                break;
            case lang('link_delete'):
                return call_user_func_array(array($this, 'delete'), $params);
                break;
            case lang('link_delete_ajax'):
                return call_user_func_array(array($this, 'delete_ajax'), $params);
                break;
            case lang('link_edit'):
                return call_user_func_array(array($this, 'edit'), $params);
                break;
            default:
                $this->index();
        }
    }

    public function index()
    {
        $this->data['types'] = $this->ticket_model->get_types();

        $this->data['title'] = "Zoznam lotérií";
        $this->master_view('user/choose-type', $this->data, 'list-tickets');
    }

    public function add($id = NULL)
    {

        if ($this->input->post('save_ticket')) {
            if ($this->input->post('type')) {
                $fname = 'save' . $this->input->post('type');
                if (method_exists($this, $fname)) {
                    $this->$fname();
                } else {
                    $this->data['error'] = 'Chyba pri ukladaní tiketu';
                }
            } else {
                $this->data['error'] = 'Chyba pri ukladaní tiketu';
            }
        }

        if (empty($id)) {
            $this->index();
        } else {

            $this->data['type'] = $this->ticket_model->get_type($id);

            if (empty($this->data['type'])) {

                $this->session->set_flashdata('error', 'Neznámy typ lotérie');
                redirect();

            }

            $this->data['user'] = $this->flexi_auth->get_user_by_identity_row_array();

            $this->data['title'] = empty($this->data['title']) ? 'Pridať tiket - ' . $this->data['type']['name'] : $this->data['title'];
            $this->master_view('user/lottery-types/' . $this->data['type']['template'], $this->data, 'add' . $this->data['type']['name']);
        }
    }

    //view filled ticket
    public function view($id = NULL)
    {

        $this->load->library('result_verification');

        if (empty($id)) {
            redirect(lang('link_account') . '/' . lang('tickets'));
        } else {
            if ($this->ticket_model->is_users_ticket($id)) {
                $this->data['view'] = true;
                $this->data['ticket_data'] = $this->ticket_model->get_ticket($id);

                $this->data['results'] = $this->result_verification->get_ticket_results($id);

                $from = $this->data['ticket_data']['date_from'];
                $when = explode(';', $this->data['ticket_data']['what_days']);
                $times = explode(';', $this->data['ticket_data']['times']);
                $type_id = $this->data['ticket_data']['type_id'];

                $this->data['result_days'] = $this->result_verification->get_all_ticket_days($from, $when, $times, $type_id);

                $this->data['type'] = $this->ticket_model->get_type($type_id);

                $this->data['title'] = 'Tiket č. ' . $this->data['ticket_data']['ticket_id'] . ' - ' . $this->data['type']['name'];
                $this->add($type_id);

            } else {
                $this->session->set_flashdata('error', 'Neexistujúci tiket');
                redirect();
            }
        }
    }

    //edit filled ticket
    public function edit($id = NULL)
    {
        if ($this->input->post('edit_ticket')) {
            if ($this->input->post('type')) {
                $fname = 'save' . $this->input->post('type');
                if (method_exists($this, $fname)) {
                    $this->$fname($id);
                } else {
                    $this->data['error'] = 'Chyba pri ukladaní tiketu';
                }
            } else {
                $this->data['error'] = 'Chyba pri ukladaní tiketu';
            }
        }

        if (empty($id)) {
            redirect(lang('link_account') . '/' . lang('tickets'));
        } else {
            if ($this->ticket_model->is_users_ticket($id)) {
                $this->data['edit'] = true;
                $this->data['ticket_data'] = $this->ticket_model->get_ticket($id);

                $this->data['title'] = 'Upraviť tiket č. ' . $this->data['ticket_data']['ticket_id'];
                $this->add($this->data['ticket_data']['type_id']);
            } else {
                $this->session->set_flashdata('error', 'Neexistujúci tiket');
                redirect();
            }
        }
    }

    //delete ticket
    public function delete($id = NULL)
    {
        if (empty($id)) {
            redirect(lang('link_account') . '/' . lang('tickets'));
        } else {
            if ($this->ticket_model->is_users_ticket($id)) {

                $this->ticket_model->delete_ticket($id);

                $this->session->set_flashdata('info', 'Tiket úspešne vymazaný');
                redirect(lang('link_account') . '/' . lang('tickets'));
            } else {
                $this->session->set_flashdata('error', 'Neexistujúci tiket');
                redirect();
            }
        }
    }

    //delete ticket
    public function delete_ajax($id = NULL)
    {
        if (!empty($id)) {
            if ($this->ticket_model->is_users_ticket($id)) {
                $this->ticket_model->delete_ticket($id);
                echo '1';
            } else {
                echo '0';
            }
        }
    }

    //save ticket type 1
    private function save1($edit = 0)
    {
        $this->load->library('result_verification');

        $type = 1;

        $this->form_validation->set_rules('tips_1', 'Stĺpec 1', 'required|exact_arr_length[6]');
        $this->form_validation->set_rules('tips_1[]', 'Stĺpec 1', 'integer');

        $columns = $this->ticket_model->get_type($type)['tip_cols'];

        for ($i = 2; $i <= $columns; $i++) {
            $this->form_validation->set_rules('tips_' . $i, 'Stĺpec ' . $i, 'exact_arr_length[6]');
            $this->form_validation->set_rules('tips_' . $i . '[]', 'Stĺpec ' . $i, 'integer');
        }

        $this->form_validation->set_rules('sance', 'Číslo Šance', 'exact_length[6]|integer');
        $this->form_validation->set_rules('when[]', 'Losovanie', 'required|integer|check_when[' . $type . ']');
        $this->form_validation->set_rules('times[]', 'Počet losovaní', 'required|integer');
        $this->form_validation->set_rules('date_from', 'Dátum prvého losovania', 'required|check_date[' . $type . ']');
        $this->form_validation->set_rules('mail_prefs', 'Odosielanie e-mailov', 'required|integer');

        if ($this->form_validation->run() == TRUE) {

            $res = $this->ticket_model->save_ticket($type, $this->result_verification->isActive(), $edit);

            if (!$res) {
                $this->data['error'] = 'Chyba pri ukladaní tiketu';
            } else {
                $this->session->set_flashdata('info', 'Tiket úspešne uložený');
                if ($edit) {
                    redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $edit);
                } else {
                    redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $res);
                }
                $this->index();
            }
        }
        return true;
    }

    //save ticket type 2
    private function save2($edit = 0)
    {
        $this->load->library('result_verification');

        $type = 2;

        $this->form_validation->set_rules('tips_1', 'Stĺpec 1', 'required|exact_arr_length[6]');
        $this->form_validation->set_rules('tips_1[]', 'Stĺpec 1', 'integer');
        $this->form_validation->set_rules('color_1[]', 'Farba v stĺpci 1', 'required|integer');

        $columns = $this->ticket_model->get_type($type)['tip_cols'];

        for ($i = 2; $i <= $columns; $i++) {
            $this->form_validation->set_rules('tips_' . $i, 'Stĺpec ' . $i, 'exact_arr_length[6]');
            $this->form_validation->set_rules('tips_' . $i . '[]', 'Stĺpec ' . $i, 'integer');

            if ($this->input->post('tips_' . $i)) {
                $this->form_validation->set_rules('color_' . $i . '[]', 'Farba v stĺpci ' . $i, 'required|integer');
            }
        }

        $this->form_validation->set_rules('sance', 'Šťastné číslo', 'exact_length[7]|integer');
        $this->form_validation->set_rules('when[]', 'Losovanie', 'required|integer|check_when[' . $type . ']');
        $this->form_validation->set_rules('times[]', 'Počet losovaní', 'required|integer');
        $this->form_validation->set_rules('date_from', 'Dátum prvého losovania', 'required|check_date[' . $type . ']');
        $this->form_validation->set_rules('mail_prefs', 'Odosielanie e-mailov', 'required|integer');

        if ($this->form_validation->run() == TRUE) {

            $res = $this->ticket_model->save_ticket($type, $this->result_verification->isActive(), $edit);

            if (!$res) {
                $this->data['error'] = 'Chyba pri ukladaní tiketu';
            } else {
                $this->session->set_flashdata('info', 'Tiket úspešne uložený');
                if ($edit) {
                    redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $edit);
                } else {
                    redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $res);
                }
                $this->index();
            }
        }
        return true;
    }

    //save ticket type 3
    private function save3($edit = 0)
    {
        $this->load->library('result_verification');

        $type = 3;

        if (!($this->input->post('tips_1') || $this->input->post('tips_2') || $this->input->post('tips_3') || $this->input->post('tips_4') || $this->input->post('tips_5'))) {
            $this->form_validation->set_rules('custom1', '', 'callback_korunka_tips');
        }

        if (($this->input->post('tips_1') || $this->input->post('tips_2') || $this->input->post('tips_3') || $this->input->post('tips_5')) && $this->input->post('tips_4')) {
            $this->form_validation->set_rules('custom2', '', 'callback_korunka_tips2');
        }

        $this->form_validation->set_rules('tips_1', 'Korunka na 3', 'exact_arr_length[3]');
        $this->form_validation->set_rules('tips_1[]', 'Korunka na 3', 'integer');

        $this->form_validation->set_rules('tips_2', 'Korunka na 4', 'exact_arr_length[4]');
        $this->form_validation->set_rules('tips_2[]', 'Korunka na 4', 'integer');

        $this->form_validation->set_rules('tips_3', 'Korunka na 5', 'exact_arr_length[5]');
        $this->form_validation->set_rules('tips_3[]', 'Korunka na 5', 'integer');

        $this->form_validation->set_rules('tips_4', 'Korunka Šestka', 'exact_arr_length[6]');
        $this->form_validation->set_rules('tips_4[]', 'Korunka Šestka', 'integer');

        if ($this->input->post('tips_5')) {
            $this->form_validation->set_rules('hoptrop[]', 'Počet čísel HopTrop', 'required|integer');
            $this->form_validation->set_rules('vklad[]', 'Vklad', 'required|integer');
            $this->form_validation->set_rules('tips_5[]', 'Korunka HopTrop', 'integer');
        }

        $this->form_validation->set_rules('times[]', 'Počet dní', 'required|integer');
        $this->form_validation->set_rules('slos[]', 'Losovanie', 'required|integer');
        $this->form_validation->set_rules('date_from', 'Dátum prvého losovania', 'required|check_date[' . $type . ']');
        $this->form_validation->set_rules('mail_prefs', 'Odosielanie e-mailov', 'required|integer');

        if ($this->form_validation->run() == TRUE) {

            if ($this->input->post('hoptrop')) {
                $hoptrop_length = $this->input->post('hoptrop', true);
                $this->form_validation->set_rules('tips_5', 'Korunka HopTrop', 'exact_arr_length[' . $hoptrop_length . ']');
            }

            if ($this->form_validation->run() == TRUE) {

                $res = $this->ticket_model->save_ticket($type, $this->result_verification->isActive(), $edit);

                if (!$res) {
                    $this->data['error'] = 'Chyba pri ukladaní tiketu';
                } else {
                    $this->session->set_flashdata('info', 'Tiket úspešne uložený');
                    if ($edit) {
                        redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $edit);
                    } else {
                        redirect(lang('link_ticket') . '/' . lang('link_view') . '/' . $res);
                    }
                    $this->index();
                }

            }

        }
        return true;
    }

    public function korunka_tips($input)
    {
        $this->form_validation->set_message('korunka_tips', 'Je potrebné vyplniť aspoň jeden tiket.');
        return FALSE;
    }

    public function korunka_tips2($input)
    {
        $this->form_validation->set_message('korunka_tips2', 'Na jednom tikete nie je možné kombinovať Korunka Šestka s ostatnými typmi Korunky.');
        return FALSE;
    }

}