<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

class MY_Controller extends CI_Controller
{
    /**
     * Class constructor with access control
     * @param bool $admin admin section
     * @param bool $restricted is access restricted?
     * @param array $exceptions pages, where no restriction will be applied
     */
    public function __construct($admin = FALSE, $restricted = TRUE, $exceptions = array())
    {
        parent::__construct();

        $this->auth = new stdClass;
        $this->load->library('flexi_auth');

        if ($restricted) {

            if (!in_array($this->uri->segment(2), $exceptions)) {

                if (!$admin) {
                    if (!$this->flexi_auth->is_logged_in()) {
                        $this->session->set_flashdata('error', lang('text_restricted'));

                        $this->session->set_userdata('last_page', current_url());

                        redirect(lang('link_auth'));
                    }
                } else {
                    if (!$this->flexi_auth->is_logged_in()) {
                        $this->session->set_flashdata('info', lang('admin_restricted'));
                        redirect('admin/auth');
                    } else if (!$this->flexi_auth->is_admin()) {
                        $this->session->set_flashdata('info', lang('admin_restricted'));
                        redirect('admin/auth');
                    }
                }
            }
        }
    }

    /**
     * Generate view for frontend with header and footer
     * @param $content_view template name to be displayed
     * @param $data data for template
     * @param string $bodyId page id
     */
    public function master_view($content_view, $data, $bodyId = 'default')
    {
        $data = $this->set_my_messages($data);

        $data['bodyId'] = $bodyId;

        $this->load->view('templates/user/header', $data);
        $this->load->view($content_view, $data);
        $this->load->view('templates/user/footer');
    }

    /**
     * Generate view for backend with header and footer
     * @param $content_view template name to be displayed
     * @param $data data for template
     * @internal param string $bodyId page id
     */
    public function admin_master_view($content_view, $data)
    {
        $data = $this->set_my_messages($data);

        $this->load->view('templates/admin/header', $data);
        $this->load->view($content_view, $data);
        $this->load->view('templates/admin/footer');
    }

    /**
     * Set messages to be displayed in view
     * @param $data
     * @return mixed
     */
    public function set_my_messages($data)
    {
        $data['message'] = (!isset($this->data['message'])) ? $this->session->flashdata('message') : $this->data['message'];
        $data['info'] = (!isset($this->data['info'])) ? $this->session->flashdata('info') : $this->data['info'];
        $data['error'] = (!isset($this->data['error'])) ? $this->session->flashdata('error') : $this->data['error'];

        return $data;
    }

}