<?php  if (!defined('BASEPATH')) exit('No direct script access allowed');

class MY_Form_validation extends CI_Form_validation
{
    public function __construct()
    {
        parent::__construct();
    }

    // Check identity is available

    protected function identity_available($identity, $user_id = FALSE)
    {
        if (!$this->CI->flexi_auth->identity_available($identity, $user_id)) {
            $status_message = $this->CI->lang->line('form_validation_duplicate_identity');
            $this->CI->form_validation->set_message('identity_available', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // Check email is available

    protected function email_available($email, $user_id = FALSE)
    {
        if (!$this->CI->flexi_auth->email_available($email, $user_id)) {
            $status_message = $this->CI->lang->line('form_validation_duplicate_email');
            $this->CI->form_validation->set_message('email_available', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // Check username is available

    protected function username_available($username, $user_id = FALSE)
    {
        if (!$this->CI->flexi_auth->username_available($username, $user_id)) {
            $status_message = $this->CI->lang->line('form_validation_duplicate_username');
            $this->CI->form_validation->set_message('username_available', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // Validate a password matches a specific users current password.

    protected function validate_current_password($current_password, $identity)
    {
        if (!$this->CI->flexi_auth->validate_current_password($current_password, $identity)) {
            $status_message = $this->CI->lang->line('form_validation_current_password');
            $this->CI->form_validation->set_message('validate_current_password', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // Validate Password

    protected function validate_password($password)
    {
        $password_length = strlen($password);
        $min_length = $this->CI->flexi_auth->min_password_length();

        // Check password length is valid and that the password only contains valid characters.
        if ($password_length >= $min_length && $this->CI->flexi_auth->valid_password_chars($password)) {
            return TRUE;
        }

        $status_message = $this->CI->lang->line('password_invalid');
        $this->CI->form_validation->set_message('validate_password', $status_message);
        return FALSE;
    }

    // Validate reCAPTCHA

    protected function validate_recaptcha()
    {
        if (!$this->CI->flexi_auth->validate_recaptcha()) {
            $status_message = $this->CI->lang->line('captcha_answer_invalid');
            $this->CI->form_validation->set_message('validate_recaptcha', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // Validate Math Captcha

    protected function validate_math_captcha($input)
    {
        if (!$this->CI->flexi_auth->validate_math_captcha($input)) {
            $status_message = $this->CI->lang->line('captcha_answer_invalid');
            $this->CI->form_validation->set_message('validate_math_captcha', $status_message);
            return FALSE;
        }
        return TRUE;
    }

    // exact array length

    protected function exact_arr_length($input, $length)
    {
        if (count($input) == $length) {
            return TRUE;
        }

        switch ($length) {
            case 1:
                $this->CI->form_validation->set_message('exact_arr_length', 'Stĺpec %s musí obsahovať presne ' . $length . ' tip.');
                break;
            case ($length > 1 && $length < 5):
                $this->CI->form_validation->set_message('exact_arr_length', 'Stĺpec %s musí obsahovať presne ' . $length . ' tipy.');
                break;
            default:
                $this->CI->form_validation->set_message('exact_arr_length', 'Stĺpec %s musí obsahovať presne ' . $length . ' tipov.');
        }
        return FALSE;
    }

    // exact length sance with spaces

    protected function exact_length_extra($input, $length)
    {
        $tmp = $this->get_result_sance($input);

        if (function_exists('mb_strlen')) {
            if (mb_strlen($tmp) != $length) {
                switch ($length) {
                    case 1:
                        $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' číslo.');
                        break;
                    case ($length > 1 && $length < 5):
                        $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' čísla.');
                        break;
                    default:
                        $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' čísel.');
                }
                return FALSE;
            } else {
                return TRUE;
            }
        }

        if (strlen($tmp) != $length) {
            switch ($length) {
                case 1:
                    $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' číslo.');
                    break;
                case ($length > 1 && $length < 5):
                    $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' čísla.');
                    break;
                default:
                    $this->CI->form_validation->set_message('exact_length_extra', 'Pole %s musí obsahovať presne ' . $length . ' čísel.');
            }
            return FALSE;
        } else {
            return TRUE;
        }

    }

    public function get_result_sance($input)
    {
        $tmp = $input;

        $arr = array();

        if (strpos($tmp, ' ')) {
            $tmp = explode(' ', $tmp);
            for ($i = 0; $i < count($tmp); $i++) {
                if (is_numeric(trim($tmp[$i]))) {
                    array_push($arr, trim($tmp[$i]));
                }
            }
            $tmp = implode('', $arr);
        }

        return $tmp;
    }

    protected function tips_count($input, $length)
    {
        $inp = $this->get_result($input);
        if (count($inp) == $length) {
            return TRUE;
        }

        switch ($length) {
            case 1:
                $this->CI->form_validation->set_message('tips_count', 'Pole %s musí obsahovať presne ' . $length . ' číslo.');
                break;
            case ($length > 1 && $length < 5):
                $this->CI->form_validation->set_message('tips_count', 'Pole %s musí obsahovať presne ' . $length . ' čísla.');
                break;
            default:
                $this->CI->form_validation->set_message('tips_count', 'Pole %s musí obsahovať presne ' . $length . ' čísel.');
        }
        return FALSE;
    }

    public function get_result($input)
    {
        $tmp = trim($input);

        $return = array();

        if (strpos($tmp, ',')) {
            $tmp = explode(',', $tmp);
            for ($i = 0; $i < count($tmp); $i++) {
                if (is_numeric(trim($tmp[$i]))) {
                    array_push($return, trim($tmp[$i]));
                }
            }
        } else if (strpos($tmp, ' ')) {
            $tmp = explode(' ', $tmp);
            for ($i = 0; $i < count($tmp); $i++) {
                if (is_numeric(trim($tmp[$i]))) {
                    array_push($return, trim($tmp[$i]));
                }
            }
        }

        return $return;
    }

    protected function check_date($input, $params)
    {
        $params = explode(';', $params);
        $type_id = $params[0];
        $admin = isset($params[1]) ? $params[1] : false;

        if (strpos($input, '.') !== false) {
            $tmp = explode('.', $input);
            if (count($tmp) == 3) {
                $input = trim($tmp[1]) . '/' . trim($tmp[0]) . '/' . trim($tmp[2]);
            }
        }

        $date = date_parse($input);

        if (checkdate($date["month"], $date["day"], $date["year"])) {

            $date = getdate(strtotime($input));
            $this->CI->load->model('ticket_model');

            $when = $this->CI->input->post('when');
            if (!$when) {
                $tmp = $this->CI->ticket_model->get_when($type_id);
                $when = explode(';', $tmp['lottery_days']);
            }

            /*if ($admin) {
                $tmp = $this->CI->ticket_model->get_when($type_id);
                $when = explode(';', $tmp['lottery_days']);
            } else {
                $when = $this->CI->input->post('when');
                if (!$when) {
                    $tmp = $this->CI->ticket_model->get_when($type_id);
                    $when = explode(';', $tmp['lottery_days']);
                }
            }*/

            //generate name days
            $tmp = getdate(strtotime('4/28/2013'));
            $what_days = array();
            for ($i = 0; $i < count($when); $i++) {
                array_push($what_days, iconv(null, "utf-8", strftime('%A', mktime(0, 0, 0, $tmp['mon'], ($tmp['mday'] + $when[$i]), $tmp['year']))));
            }

            if (in_array($date['wday'], $when)) {
                return TRUE;
            } else {
                $this->CI->form_validation->set_message('check_date', 'Neplatný dátum v poli %s. Musí byť niektorý z týchto dní: ' . implode(', ', $what_days));
                return FALSE;
            }

        }

        $this->CI->form_validation->set_message('check_date', 'Neplatný dátum v poli %s.');
        return FALSE;
    }

    protected function check_when($input, $type_id)
    {
        $this->CI->load->model('ticket_model');
        $tmp = $this->CI->ticket_model->get_when($type_id);
        $when = explode(';', $tmp['lottery_days']);
        if (in_array($input, $when))
            return TRUE;

        $this->CI->form_validation->set_message('check_when', 'Neplatný deň v poli %s.');
        return FALSE;
    }

}

/* End of file MY_Form_validation.php */
/* Location: ./application/library/MY_Form_validation.php */ 	