<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Class Auth_model
 * @property Flexi_auth flexi_auth
 */
class Auth_model extends CI_Model
{

    // The following method prevents an error occurring when $this->data is modified.
    // Error Message: 'Indirect modification of overloaded property Demo_cart_admin_model::$data has no effect'.
    public function &__get($key)
    {
        $CI =& get_instance();
        return $CI->$key;
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Login
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * login
     * Validate the submitted login details and attempt to log the user into their account.
     */
    function login()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        $this->form_validation->set_rules('login_identity', 'E-mail', 'required|valid_email');
        $this->form_validation->set_rules('login_password', 'Heslo', 'required');
//        $this->form_validation->set_rules('remember_me');

        // If failed login attempts from users IP exceeds limit defined by config file, validate captcha.
        if ($this->flexi_auth->ip_login_attempts_exceeded()) {
            /**
             * reCAPTCHA
             * http://www.google.com/recaptcha
             * To activate reCAPTCHA, ensure the 'recaptcha_response_field' validation below is uncommented and then comment out the 'login_captcha' validation further below.
             *
             * The custom validation rule 'validate_recaptcha' can be found in '../libaries/MY_Form_validation.php'.
             * The form field name used by 'reCAPTCHA' is 'recaptcha_response_field', this field name IS NOT editable.
             *
             * Note: To use this example, you will also need to enable the recaptcha examples in 'controllers/auth.php', and 'views/demo/login_view.php'.
             */
            $this->form_validation->set_rules('recaptcha_response_field', 'Captcha Answer', 'required|validate_recaptcha');

            /**
             * flexi auths math CAPTCHA
             * Math CAPTCHA is a basic CAPTCHA style feature that asks users a basic maths based question to validate they are indeed not a bot.
             * To activate Math CAPTCHA, ensure the 'login_captcha' validation below is uncommented and then comment out the 'recaptcha_response_field' validation above.
             *
             * The field value submitted as the answer to the math captcha must be submitted to the 'validate_math_captcha' validation function.
             * The custom validation rule 'validate_math_captcha' can be found in '../libaries/MY_Form_validation.php'.
             *
             * Note: To use this example, you will also need to enable the math_captcha examples in 'controllers/auth.php', and 'views/demo/login_view.php'.
             */
            # $this->form_validation->set_rules('login_captcha', 'Captcha Answer', 'required|validate_math_captcha['.$this->input->post('login_captcha').']');
        }

        // Run the validation.
        if ($this->form_validation->run()) {

            // Verify login data.
            $logged = $this->flexi_auth->login($this->input->post('login_identity'), $this->input->post('login_password'));

            if (!$logged) {
                $this->data['message'] = $this->flexi_auth->get_messages();
            } else {
                // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
                $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

                // Reload page, if login was successful, sessions will have been created that will then further redirect verified users.

                if ($this->session->userdata('last_page')) {
                    $redir = $this->session->userdata('last_page');
                    $this->session->unset_userdata('last_page');
                    redirect($redir);
                } else {
                    redirect();
                }
            }

        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }


    function adminLogin()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        $this->form_validation->set_rules('login_identity', 'E-mail', 'required');
        $this->form_validation->set_rules('login_password', 'Heslo', 'required');

        // If failed login attempts from users IP exceeds limit defined by config file, validate captcha.
        if ($this->flexi_auth->ip_login_attempts_exceeded()) {
            /**
             * reCAPTCHA
             * http://www.google.com/recaptcha
             * To activate reCAPTCHA, ensure the 'recaptcha_response_field' validation below is uncommented and then comment out the 'login_captcha' validation further below.
             *
             * The custom validation rule 'validate_recaptcha' can be found in '../libaries/MY_Form_validation.php'.
             * The form field name used by 'reCAPTCHA' is 'recaptcha_response_field', this field name IS NOT editable.
             *
             * Note: To use this example, you will also need to enable the recaptcha examples in 'controllers/auth.php', and 'views/demo/login_view.php'.
             */
            $this->form_validation->set_rules('recaptcha_response_field', 'Captcha Answer', 'required|validate_recaptcha');

            /**
             * flexi auths math CAPTCHA
             * Math CAPTCHA is a basic CAPTCHA style feature that asks users a basic maths based question to validate they are indeed not a bot.
             * To activate Math CAPTCHA, ensure the 'login_captcha' validation below is uncommented and then comment out the 'recaptcha_response_field' validation above.
             *
             * The field value submitted as the answer to the math captcha must be submitted to the 'validate_math_captcha' validation function.
             * The custom validation rule 'validate_math_captcha' can be found in '../libaries/MY_Form_validation.php'.
             *
             * Note: To use this example, you will also need to enable the math_captcha examples in 'controllers/auth.php', and 'views/demo/login_view.php'.
             */
            # $this->form_validation->set_rules('login_captcha', 'Captcha Answer', 'required|validate_math_captcha['.$this->input->post('login_captcha').']');
        }

        // Run the validation.
        if ($this->form_validation->run()) {
            $isAdmin = TRUE;

            // Verify login data.
            $logged = $this->flexi_auth->login($this->input->post('login_identity'), $this->input->post('login_password'), null, $isAdmin);

            if (!$logged) {
                $this->data['message'] = $this->flexi_auth->get_messages();
            } else {
                // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
                $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

                // Reload page, if login was successful, sessions will have been created that will then further redirect verified users.
                redirect('admin');
            }

        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    /**
     * login_via_ajax
     * Attempt to log a user in via ajax.
     * This example is a much more simplified version of the above 'login' function example as it just returns a boolean value of
     * whether or not the submitted details successfully logged a user in - no redirects or status messages are set.
     */
    function login_via_ajax()
    {
        $remember_user = ($this->input->post('remember_me') == 1);

        // Verify login data.
        return $this->flexi_auth->login($this->input->post('login_identity'), $this->input->post('login_password'), $remember_user);
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Account Registration
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * register_account
     * Create a new user account.
     * Then if defined via the '$instant_activate' var, automatically log the user into their account.
     */
    function register_account()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        // The custom rules 'identity_available' and 'validate_password' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'register_email_address', 'label' => 'E-mail', 'rules' => 'required|valid_email|identity_available|max_length[100]'),
            array('field' => 'register_username', 'label' => 'Užívateľské meno', 'rules' => 'required|min_length[4]|max_length[17]'),
            array('field' => 'register_password', 'label' => 'Heslo', 'rules' => 'required|validate_password|max_length[100]'),
            array('field' => 'register_confirm_password', 'label' => 'Potvrdiť heslo', 'rules' => 'required|matches[register_password]')
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            // Get user login details from input.
            $email = $this->input->post('register_email_address');
            $username = $this->input->post('register_username');
            $password = $this->input->post('register_password');

            // Set whether to instantly activate account.
            // This var will be used twice, once for registration, then to check if to log the user in after registration.
            $instant_activate = FALSE;

            // The last 2 variables on the register function are optional, these variables allow you to:
            // #1. Specify the group ID for the user to be added to (i.e. 'Moderator' / 'Public'), the default is set via the config file.
            // #2. Set whether to automatically activate the account upon registration, default is FALSE.
            // Note: An account activation email will be automatically sent if auto activate is FALSE, or if an activation time limit is set by the config file.
            $response = $this->flexi_auth->insert_user($email, $username, $password, null, 1, $instant_activate);

            if ($response) {
                // This is an example 'Welcome' email that could be sent to a new user upon registration.
                // Bear in mind, if registration has been set to require the user activates their account, they will already be receiving an activation email.
                // Therefore sending an additional email welcoming the user may be deemed unnecessary.
                $email_data = array('identity' => $email);
                $this->flexi_auth->send_email($email, 'Vitajte', 'registration_welcome.tpl.php', $email_data);
                // Note: The 'registration_welcome.tpl.php' template file is located in the '../views/includes/email/' directory defined by the config file.

                ###+++++++++++++++++###

                // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
                $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

                // This is an example of how to log the user into their account immeadiately after registering.
                // This example would only be used if users do not have to authenticate their account via email upon registration.
                if ($instant_activate && $this->flexi_auth->login($email, $password)) {
                    // Redirect user to public dashboard.
                    redirect();
                }

                // Redirect user to login page
                redirect(lang('link_auth'));
            }
        }

        // Set validation errors.
        $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

        return FALSE;
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Account Activation
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * resend_activation_token
     * Resends a new account activation token to a users email address.
     */
    function resend_activation_token()
    {
        $this->load->library('form_validation');

        $this->form_validation->set_rules('activation_token_identity', 'E-mail', 'required|valid_email');

        // Run the validation.
        if ($this->form_validation->run()) {
            // Verify identity and resend activation token.
            $response = $this->flexi_auth->resend_activation_token($this->input->post('activation_token_identity'));

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            // Redirect user.
            ($response) ? redirect(lang('link_auth')) : redirect(lang('link_auth') . '/' . lang('link_resend_activation_token'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Reseting Passwords
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * forgotten_password
     * Sends a 'Forgotten Password' email to a users email address.
     * The email will contain a link that redirects the user to the site, a token within the link is verified, and the user can then manually reset their password.
     */
    function forgotten_password()
    {
        $this->load->library('form_validation');

        $this->form_validation->set_rules('forgot_password_identity', 'E-mail', 'required|valid_email');

        // Run the validation.
        if ($this->form_validation->run()) {
            // The 'forgotten_password()' function will verify the users identity exists and automatically send a 'Forgotten Password' email.
            $response = $this->flexi_auth->forgotten_password($this->input->post('forgot_password_identity'));

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            // Redirect user.
            ($response) ? redirect(lang('link_auth')) : redirect(lang('link_auth') . '/' . lang('link_forgotten_password'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    /**
     * manual_reset_forgotten_password
     * This example lets the user manually reset their password rather than automatically sending them a new random password via email.
     * The function validates the user via a token within the url of the current site page, then validates their current and newly submitted passwords are valid.
     */
    function manual_reset_forgotten_password($user_id, $token)
    {
        $this->load->library('form_validation');

        // Set validation rules
        // The custom rule 'validate_password' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'new_password', 'label' => 'Nové heslo', 'rules' => 'required|validate_password|matches[confirm_new_password]|max_length[100]'),
            array('field' => 'confirm_new_password', 'label' => 'Potvrdiť nové heslo', 'rules' => 'required')
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            // Get password data from input.
            $new_password = $this->input->post('new_password');

            // The 'forgotten_password_complete()' function is used to either manually set a new password, or to auto generate a new password.
            // For this example, we want to manually set a new password, so ensure the 3rd argument includes the $new_password var, or else a  new password.
            // The function will then validate the token exists and set the new password.
            $this->flexi_auth->forgotten_password_complete($user_id, $token, $new_password);

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            redirect(lang('link_auth'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###
    // Manage User Account
    ###++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++###

    /**
     * update_account
     * Updates a users account and profile data.
     */
    function update_account()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        // The custom rule 'identity_available' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'update_username', 'label' => 'Užívateľské meno', 'rules' => 'required|min_length[4]|max_length[17]')
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            // Note: This example requires that the user updates their email address via a separate page for verification purposes.

            // Get user id from session to use in the update function as a primary key.
            $user_id = $this->flexi_auth->get_user_id();

            // Get user profile data from input.
            // IMPORTANT NOTE: As we are updating multiple tables (The main user account and user profile tables), it is very important to pass the
            // primary key column and value in the $profile_data for any custom user tables being updated, otherwise, the function will not
            // be able to identify the correct custom data row.
            // In this example, the primary key column and value is 'upro_uacc_fk' => $user_id.
            $profile_data = array(
                'upro_uacc_fk' => $user_id,
                $this->flexi_auth->db_column('user_acc', 'username') => $this->input->post('update_username')
            );

            // If we were only updating profile data (i.e. no email or username included), we could use the 'update_custom_user_data()' function instead.
            $response = $this->flexi_auth->update_user($user_id, $profile_data);

            if ($response) {
                $newdata = $this->session->userdata($this->auth->session_name['name']);
                $newdata['username'] = $this->input->post('update_username');
                $this->session->set_userdata(array($this->auth->session_name['name'] => $newdata));
            }

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            // Redirect user.
            ($response) ? redirect(lang('link_account')) : redirect(lang('link_account') . '/' . lang('link_edit'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    /**
     * change_password
     * Updates a users password.
     */
    function change_password()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        // The custom rule 'validate_password' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'current_password', 'label' => 'Súčasné heslo', 'rules' => 'required'),
            array('field' => 'new_password', 'label' => 'Nové heslo', 'rules' => 'required|validate_password|matches[confirm_new_password]|max_length[100]'),
            array('field' => 'confirm_new_password', 'label' => 'Potvrdiť heslo', 'rules' => 'required')
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            // Get password data from input.
            $identity = $this->flexi_auth->get_user_identity();
            $current_password = $this->input->post('current_password');
            $new_password = $this->input->post('new_password');

            // Note: Changing a password will delete all 'Remember me' database sessions for the user, except their current session.
            $response = $this->flexi_auth->change_password($identity, $current_password, $new_password);

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            // Redirect user.
            // Note: As an added layer of security, you may wish to email the user that their password has been updated.
            ($response) ? redirect(lang('link_account')) : redirect(lang('link_account') . '/' . lang('link_edit'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    function change_prefs()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        // The custom rule 'validate_password' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'mail_prefs', 'label' => 'Email preferences', 'rules' => 'required|integer')
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            // Get password data from input.
            $id = $this->flexi_auth->get_user_id();

            if ($id == null) return false;

            $this->db->set('email_prefs', $this->input->post('mail_prefs'));
            $this->db->where('uacc_id', $id);

            $response = $this->db->update('user_accounts');

            if ($response)
                $this->session->set_flashdata('info', 'Nastavenie odosielania e-mailov úspešne zmenené.');

            // Redirect user.
            ($response) ? redirect(lang('link_account')) : redirect(lang('link_account') . '/' . lang('link_edit'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    /**
     * send_new_email_activation
     * This demo has 2 methods of updating a logged in users email address.
     * The first option simply allows the users to change their email address along with the rest of their account data via entering it into a form fields.
     * The second option requires users to verify their email address via clicking a link that is sent to that same email address.
     * The purpose of the second option is to prevent users entering a mispelt email address, which would then prevent the user from logging back in.
     */
    function send_new_email_activation()
    {
        $this->load->library('form_validation');

        // Set validation rules.
        // The custom rule 'identity_available' can be found in '../libaries/MY_Form_validation.php'.
        $validation_rules = array(
            array('field' => 'email_address', 'label' => 'Nová e-mailová adresa', 'rules' => 'required|valid_email|identity_available'),
        );

        $this->form_validation->set_rules($validation_rules);

        // Run the validation.
        if ($this->form_validation->run()) {
            $user_id = $this->flexi_auth->get_user_id();

            // The 'update_email_via_verification()' function generates a verification token that is then emailed to the user.
            $this->flexi_auth->update_email_via_verification($user_id, $this->input->post('email_address'));

            // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
            $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

            redirect(lang('link_account'));
        } else {
            // Set validation errors.
            $this->data['message'] = validation_errors('<p class="error_msg">', '</p>');

            return FALSE;
        }
    }

    /**
     * verify_updated_email
     * Verifies a token within the current url and updates a users email address.
     */
    function verify_updated_email($user_id, $token)
    {
        // Verify the update email token and if valid, update their email address.
        $this->flexi_auth->verify_updated_email($user_id, $token);

        // Save any public status or error messages (Whilst suppressing any admin messages) to CI's flash session data.
        $this->session->set_flashdata('message', $this->flexi_auth->get_messages());

        // Redirect user.
        // Logged in users are redirected to the restricted public user dashboard, otherwise the user is redirected to the login page.
        if ($this->flexi_auth->is_logged_in()) {
            redirect(lang('link_account'));
        } else {
            redirect(lang('link_auth'));
        }
    }

}
/* End of file demo_auth_model.php */
/* Location: ./application/models/demo_auth_model.php */