<?php if (!defined('BASEPATH')) exit('No direct script access allowed');

/**
 * Class Ticket_model
 *
 * @property Result_verification result_verification
 */
class Ticket_model extends CI_Model
{

    function __construct()
    {
        parent::__construct();
    }

    /**
     * Vrati typy loterii a info o loterii
     * @return mixed
     */
    function get_types()
    {
        $query = $this->db->get('ticket_types');
        return $query->result_array();
    }

    /**
     * Vrati info o loterii podla id
     * @param $id typ loterie
     * @return mixed
     */
    function get_type($id)
    {
        $query = $this->db->get_where('ticket_types', array('type_id' => $id), 1);
        return $query->row_array();
    }

    /**
     * Vrati dni kedy sa losuje loteria
     * @param $type_id typ loteri
     * @return mixed
     */
    function get_when($type_id)
    {
        $this->db->select('lottery_days');
        $this->db->from('ticket_types');
        $this->db->where('type_id', $type_id);
        $query = $this->db->get();
        return $query->row_array();
    }


    public function ticket_count()
    {
        return $this->db->count_all("Country");
    }


    function save_ticket($type, $active, $edit = 0)
    {
        if ($type == null) return false;

        //save info to ticket_list table
        if (!$edit) {
            $this->db->set('uacc_id', $this->flexi_auth->get_user_id());
            $this->db->set('type_id', $type);
        }

        $this->db->set('active', $active);
        $this->db->set('added', date("Y-m-d H:i:s"));

        $input = $this->input->post('date_from');
        if (strpos($input, '.') !== false) {
            $tmp = explode('.', $input);
            if (count($tmp) == 3) {
                $input = trim($tmp[1]) . '/' . trim($tmp[0]) . '/' . trim($tmp[2]);
            }
        }

        $date = date_parse($input);
        $this->db->set('date_from', date("Y-m-d", mktime(0, 0, 0, $date['month'], $date['day'], $date['year'])));

        if ($this->input->post('when') !== false) {
            $this->db->set('what_days', implode(";", $this->input->post('when')));
        } else { //plati pre korunku
            $this->db->set('what_days', '0;1;2;3;4;5;6');
        }

        if ($this->input->post('times') !== false) {
//            $this->db->set('times', implode(";", $this->input->post('times')));
            $this->db->set('times', $this->input->post('times'));
        }

//        $data['user'] = $this->flexi_auth->get_user_by_identity_row_array();

//        if ($this->input->post('mail_prefs') != $data['user']['email_prefs']) {
        $this->db->set('mail_prefs', $this->input->post('mail_prefs'));
//        }

        if ($edit) {
            $this->db->where('ticket_id', $edit);
            $q1 = $this->db->update('ticket_list');
        } else {
            $q1 = $this->db->insert('ticket_list');
        }

        //save or update info to ticket_tips table
        if ($q1) {
            if (!$edit) {
                $ins_id = $this->db->insert_id();
                $this->db->set('ticket_id', $ins_id);
            }

            $columns = $this->ticket_model->get_type($type)['tip_cols'];
            $tips = array();

            for ($ix = 1; $ix <= $columns; $ix++) {
                $tips[$ix]['tip'] = ($this->input->post('tips_' . $ix) != null) ? $this->input->post('tips_' . $ix) : array();
                $tips[$ix]['color'] = array();

                if ($this->input->post('color_' . $ix) !== false && !empty($tips[$ix]['tip'])) { //loto
                    $tips[$ix]['color'] = $this->input->post('color_' . $ix);
                }
            }

            $this->db->set('tip', json_encode($tips));

            if ($this->input->post('sance') !== false) { //sportka, loto
                $this->db->set('additional', $this->input->post('sance') == '' ? null : $this->input->post('sance'));
            }

            if ($type == 3) { //korunka, typ, slosovani, hoptrop
                $additional = array();
                $additional['slos'] = array();
                $additional['hoptrop'] = null;
                $additional['vklad'] = array();

                if ($this->input->post('slos') !== false) { //korunka
                    $additional['slos'] = $this->input->post('slos');
//                    array_push($additional, implode(',', $this->input->post('slos')));
                }

                if ($this->input->post('tips_5')) { //vyplneny stlpec hoptrop
                    if ($this->input->post('hoptrop') !== false) { //korunka, pocet cisel
                        $additional['hoptrop'] = $this->input->post('hoptrop');
                        array_push($additional, array_shift($this->input->post('hoptrop')));
                    }

                    if ($this->input->post('vklad') !== false) { //korunka
                        $additional['vklad'] = $this->input->post('vklad');
//                        array_push($additional, implode(',', $this->input->post('vklad')));
                    }
                }

//                $this->db->set('additional', implode(';', $additional));
                $this->db->set('additional', json_encode($additional));
            }


            if ($edit) {
                $this->db->where('ticket_id', $edit);
                $r = $this->db->update('ticket_tips');
                if ($r)
                    return $edit;
                else
                    return false;
            } else {
                $r = $this->db->insert('ticket_tips');
                if ($r)
                    return $ins_id;
                else
                    return false;
            }

        } else {
            return false;
        }
    }

    /**
     * Deaktivuje tiket
     * @param $ticket_id
     * @return bool|object
     */
    function deactivate_ticket($ticket_id)
    {
        if ($ticket_id == null) return false;

        $this->db->set('active', 0);

        $this->db->where('ticket_id', $ticket_id);
        return $this->db->update('ticket_list');
    }

    /**
     * Vrati tikety prihlaseneho usera
     * @param bool $count ak true, vrati pocet
     * @param int $limit
     * @param int $start
     * @param int $type typ loterie
     * @return string
     */
    function get_tickets($count = false, $limit = 0, $start = 0, $type = 0)
    {
        if ($count) {
            $this->db->select('uacc_id');
            $this->db->from('ticket_list');
            $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
            if ($type != 0) $this->db->where('type_id', $type);
            return $this->db->count_all_results();
        } else {
            $this->db->select('*');
            $this->db->from('ticket_list');
            $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
            if ($type != 0) $this->db->where('type_id', $type);
            $this->db->join('ticket_types', 'ticket_types.type_id = ticket_list.type_id');
            $this->db->order_by('added', 'desc');
            if ($limit || $start) {
                $this->db->limit($limit, $start);
            }
            $query = $this->db->get();
            return $query->result_array();
        }
    }

    /**
     * Zisti ci tiket patri prihlasenemu userovi
     * @param $id
     * @return bool
     */
    function is_users_ticket($id)
    {
        $this->db->select('ticket_id');
        $this->db->from('ticket_list');
        $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
        $this->db->where('ticket_id', $id);
        $this->db->limit(1);
        return ($this->db->count_all_results() == 1) ? true : false;
    }

    /**
     * Vrati info o tikete a tipy patriace tiketu podla id
     * @param $id
     * @return mixed
     */
    function get_ticket($id)
    {
        $this->db->select('*');
        $this->db->from('ticket_list');
        $this->db->where('ticket_list.ticket_id', $id);
        $this->db->join('ticket_tips', 'ticket_tips.ticket_id = ticket_list.ticket_id');
        $this->db->limit(1);
        $query = $this->db->get();
        return $query->row_array();
    }

    /**
     * Vymaze tiket podla id
     * @param $id
     */
    function delete_ticket($id)
    {
        $tables = array('ticket_list', 'ticket_tips');
        $this->db->where('ticket_id', $id);
        $this->db->delete($tables);
    }

    /**
     * Ulozi alebo edituje vyledok losovania loterie
     * @param $type typ loterie
     * @param int $edit id vysledku, ktore s ma editovat
     * @return bool|object
     */
    function save_result($type, $edit = 0)
    {
        if ($type == null) return false;

        $additional = array();

        //save info to ticket_list table
        $tah1 = $this->form_validation->get_result($this->input->post('tah1'));
        $tah1 = implode(',', $tah1);
        $this->db->set('tah1', $tah1);

        if ($this->input->post('tah2') !== false) {
            $tah2 = $this->form_validation->get_result($this->input->post('tah2'));
            $tah2 = implode(',', $tah2);
            $this->db->set('tah2', $tah2);
        }

        if ($this->input->post('color') !== false) {
            $this->db->set('tah2', implode(';', $this->input->post('color')));
        }

        if ($this->input->post('slos') !== false) {
            $this->db->set('order', array_shift($this->input->post('slos')));
        }

        if ($this->input->post('sance') !== false) {
            $sance = $this->form_validation->get_result_sance($this->input->post('sance'));
            array_push($additional, $sance);
        }

        $input = $this->input->post('date');
        if (strpos($input, '.') !== false) {
            $tmp = explode('.', $input);
            if (count($tmp) == 3) {
                $input = trim($tmp[1]) . '/' . trim($tmp[0]) . '/' . trim($tmp[2]);
            }
        }

        $date = date_parse($input);
        $this->db->set('lottery_date', date("Y-m-d", mktime(0, 0, 0, $date['month'], $date['day'], $date['year'])));

        $tmp = getdate(strtotime($input));
        $this->db->set('day_in_week', $tmp['wday']);

        if ($this->input->post('mimoradna') !== false) {
            array_push($additional, $this->input->post('mimoradna'));
        }

        if ($this->input->post('info') !== false) {
            $this->db->set('info', $this->input->post('info'));
        }

        if (!empty($additional)) {
            $this->db->set('additional', implode(";", $additional));
        }

        $this->db->set('type_id', $type);

        if ($edit) {
            $this->db->where('result_id', $edit);
            return $this->db->update('ticket_results');
        } else {
            return $this->db->insert('ticket_results');
        }

    }


    /**
     * Vrati vysledky losovania pre danu loteriu
     * @param $type typ loterie
     * @param bool $count ak true, vrati pocet
     * @param int $limit
     * @param int $start
     * @return string
     */
    function get_results($type, $count = false, $limit = 0, $start = 0)
    {
        if ($count) {
            $this->db->select('result_id');
            $this->db->from('ticket_results');
            $this->db->where('type_id', $type);
            return $this->db->count_all_results();
        } else {
            $this->db->select('*');
            $this->db->from('ticket_results');
            $this->db->where('type_id', $type);
            $this->db->order_by('lottery_date', 'desc');
            $this->db->order_by('order', 'desc');
            if ($limit || $start) {
                $this->db->limit($limit, $start);
            }
            $query = $this->db->get();
            return $query->result_array();
        }
    }

    /**
     * Vrati vysledky losovania pre danu loteriu. Data su urcene pre ajax tabulku.
     */
    function get_results_ajax($type)
    {
        if ($type == 3) {
            $aColumns2 = array('result_id', 'tah1', 'tah2', '`order`', 'info', 'lottery_date');
            $aColumns = array('result_id', 'tah1', 'tah2', 'order', 'info', 'lottery_date');
        } else {
            $aColumns2 = array('result_id', 'tah1', 'tah2', 'additional', 'info', 'lottery_date');
            $aColumns = array('result_id', 'tah1', 'tah2', 'additional', 'info', 'lottery_date');
        }

        $sTable = 'ticket_results';

        $iDisplayStart = $this->input->get_post('iDisplayStart', true);
        $iDisplayLength = $this->input->get_post('iDisplayLength', true);
        $iSortCol_0 = $this->input->get_post('iSortCol_0', true);
        $iSortingCols = $this->input->get_post('iSortingCols', true);
        $sSearch = $this->input->get_post('sSearch', true);
        $sEcho = $this->input->get_post('sEcho', true);

        // Paging
        if (isset($iDisplayStart) && $iDisplayLength != '-1') {
            $this->db->limit($this->db->escape_str($iDisplayLength), $this->db->escape_str($iDisplayStart));
        }

        // Ordering
        if (isset($iSortCol_0)) {
            for ($i = 0; $i < intval($iSortingCols); $i++) {
                $iSortCol = $this->input->get_post('iSortCol_' . $i, true);
                $bSortable = $this->input->get_post('bSortable_' . intval($iSortCol), true);
                $sSortDir = $this->input->get_post('sSortDir_' . $i, true);

                if ($bSortable == 'true') {
                    $this->db->order_by($aColumns[intval($this->db->escape_str($iSortCol))], $this->db->escape_str($sSortDir));
                }
            }
        }

        /*
         * Filtering
         * NOTE this does not match the built-in DataTables filtering which does it
         * word by word on any field. It's possible to do here, but concerned about efficiency
         * on very large tables, and MySQL's regex functionality is very limited
         */
        if (isset($sSearch) && !empty($sSearch)) {
            $where = "type_id = '" . $type . "' AND (";
            $or = false;
            for ($i = 0; $i < count($aColumns); $i++) {
                $bSearchable = $this->input->get_post('bSearchable_' . $i, true);

                // Individual column filtering
                if (isset($bSearchable) && $bSearchable == 'true') {
                    if (!$or) {
                        $where .= "`" . $aColumns[$i] . "` LIKE '%" . $this->db->escape_like_str($sSearch) . "%'";
                    } else {
                        $where .= " OR `" . $aColumns[$i] . "` LIKE '%" . $this->db->escape_like_str($sSearch) . "%'";
                    }
                    $or = true;
//                    $this->db->or_like($aColumns[$i], $this->db->escape_like_str($sSearch));
                }
            }
            $where .= ") ";
            $this->db->where($where);
        } else {
            $this->db->where('type_id', $type);
        }

        // Select Data
        $this->db->select('SQL_CALC_FOUND_ROWS ' . str_replace(' , ', ' ', implode(', ', $aColumns2)), false);
        $rResult = $this->db->get($sTable);
        $tmp = $this->db->last_query();

        // Data set length after filtering
        $this->db->select('FOUND_ROWS() AS found_rows');
        $iFilteredTotal = $this->db->get()->row()->found_rows;

        // Total data set length
        $this->db->select('result_id');
        $this->db->from('ticket_results');
        $this->db->where('type_id', $type);
        $iTotal = $this->db->count_all_results();

        // Output
        $output = array(
            'sEcho' => intval($sEcho),
            'iTotalRecords' => $iTotal,
            'iTotalDisplayRecords' => $iFilteredTotal,
            'aaData' => array()
        );

        $this->load->library('result_verification');

        foreach ($rResult->result_array() as $aRow) {
            $row = array();

            foreach ($aColumns as $col) {
                if ($col == 'lottery_date') {
                    $tmp = strtotime($aRow[$col]);

                    if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
                        $row[] = iconv('', 'utf-8', strftime("%a, %#d. %B %Y", $tmp));
                    } else {
                        $row[] = iconv('', 'utf-8', strftime("%a, %e. %B %Y", $tmp));
                    }
                } else if ($col == 'info') {
                    $string = substr(strip_tags($aRow[$col]), 0, 20) . '...';
                    $row[] = $string;
                } else if ($col == 'additional') {
                    $tmp = explode(';', $aRow[$col]);
                    $sance = isset($tmp[0]) ? $tmp[0] : '';
                    $row[] = $sance;
                } else if ($type == 2 && $col == 'tah2') {
                    if ($aRow[$col] == 1) {
                        $row[] = 'Biela';
                    } else if ($aRow[$col] == 2) {
                        $row[] = 'Čierna';
                    }
                } else if ($type == 3 && $col == 'order') {
                    if ($aRow[$col] == 1) {
                        $row[] = '14:00';
                    } else if ($aRow[$col] == 2) {
                        $row[] = '18:00';
                    }
                } else {
                    $row[] = $aRow[$col];
                }
            }

            $row[] = anchor('admin/view_result/' . $aRow['result_id'], '<img src=' . asset_img() . 'view.png >', 'title="Zobraziť tiket"');
            $row[] = anchor('admin/edit_result/' . $aRow['result_id'], '<img src=' . asset_img() . 'edit.png >', 'title="Upraviť tiket"');
            $row[] = anchor('admin/delete_result_ajax/' . $aRow['result_id'], '<img src=' . asset_img() . 'delete.png >', array('title' => 'Odstrániť tiket',
                'class' => 'deleteicon'));

            $output['aaData'][] = $row;
        }

        echo json_encode($output);
    }

    /**
     * Vrati vysledok losovania loterie podla id
     * @param $id id vysledku
     * @return mixed
     */
    function get_result($id)
    {
        $this->db->select('*');
        $this->db->from('ticket_results');
        $this->db->where('result_id', $id);
        $this->db->limit(1);
        $query = $this->db->get();
        return $query->row_array();
    }

    /**
     * Vrati vysledky loterie od zadaneho dna
     * @param $from_day od akeho dna sa maju hladat vysledky
     * @param $type typ loterie
     * @param $days_in_week dni v tyzdni kedy sa maju hladat vysledky loterie
     * @param $limit
     * @return mixed
     */
    function get_results_from_day($from_day, $type, $days_in_week, $limit)
    {
        $this->db->select('*');
        $this->db->from('ticket_results');
        $this->db->where('type_id', $type);
        $today = date("Y-m-d", mktime());
        $this->db->where("lottery_date BETWEEN CAST('" . $from_day . "' AS DATE) AND CAST('" . $today . "' AS DATE)");
        $this->db->where_in('day_in_week', $days_in_week);
        $this->db->order_by('lottery_date asc, order asc');
        $this->db->limit($limit);
        $query = $this->db->get();
        return $query->result_array();
    }

    /**
     * Vymaze vysledok loterie podla id
     * @param $id id vysledku
     */
    function delete_result($id)
    {
        $tables = array('ticket_results');
        $this->db->where('result_id', $id);
        $this->db->delete($tables);
    }

    /**
     * vrati vsetky tikety, ktore su aktivne a maju nastaveny den losovania na den, ktory sa prave kontroluje
     *
     * @param $type_id typ loterie
     * @param $day den v tyzdni
     * @param $lot_day den zlosovania
     * @return mixed
     */
    function get_active_tickets_by_type_and_day($type_id, $day, $lot_day)
    {
        $this->db->select('ticket_list.*, user_accounts.uacc_email, user_accounts.uacc_username, user_accounts.uacc_active,
         user_accounts.uacc_suspend, user_accounts.uacc_id, user_accounts.email_prefs, ticket_tips.*, ticket_types.name, ticket_types.company, ticket_types.template');
        $this->db->from('ticket_list');
        $this->db->where('ticket_list.active', 1);
        $this->db->where('ticket_list.type_id', $type_id);
        $this->db->where("date_from <= CAST('" . $lot_day . "' AS DATE)");
        $this->db->like('ticket_list.what_days', $day);
        $this->db->join('user_accounts', 'ticket_list.uacc_id = user_accounts.uacc_id');
        $this->db->join('ticket_tips', 'ticket_list.ticket_id = ticket_tips.ticket_id');
        $this->db->join('ticket_types', 'ticket_types.type_id = ticket_list.type_id');
        $this->db->order_by('user_accounts.uacc_id asc');
        $query = $this->db->get();
        return $query->result_array();
    }

    /**
     * Vrati tikety pre prihlaseneho uzivatela. Data su urcene pre ajax tabulku.
     */
    function get_tickets_ajax()
    {
//        $aColumns = array('ticket_list.ticket_id', 'ticket_types.name', 'ticket_types.company', 'ticket_list.active', 'ticket_list.added', 'ticket_list.date_from');
        $aColumns = array('ticket_id', 'name', 'company', 'active', 'winning', 'added', 'date_from');

        $sTable = 'ticket_list';

        $iDisplayStart = $this->input->get_post('iDisplayStart', true);
        $iDisplayLength = $this->input->get_post('iDisplayLength', true);
        $iSortCol_0 = $this->input->get_post('iSortCol_0', true);
        $iSortingCols = $this->input->get_post('iSortingCols', true);
        $sSearch = $this->input->get_post('sSearch', true);
        $sEcho = $this->input->get_post('sEcho', true);

        // Paging
        if (isset($iDisplayStart) && $iDisplayLength != '-1') {
            $this->db->limit($this->db->escape_str($iDisplayLength), $this->db->escape_str($iDisplayStart));
        }

        // Ordering
        if (isset($iSortCol_0)) {
            for ($i = 0; $i < intval($iSortingCols); $i++) {
                $iSortCol = $this->input->get_post('iSortCol_' . $i, true);
                $bSortable = $this->input->get_post('bSortable_' . intval($iSortCol), true);
                $sSortDir = $this->input->get_post('sSortDir_' . $i, true);

                if ($bSortable == 'true') {
                    $this->db->order_by($aColumns[intval($this->db->escape_str($iSortCol))], $this->db->escape_str($sSortDir));
                }
            }
        }

        /*
         * Filtering
         * NOTE this does not match the built-in DataTables filtering which does it
         * word by word on any field. It's possible to do here, but concerned about efficiency
         * on very large tables, and MySQL's regex functionality is very limited
         */
        if (isset($sSearch) && !empty($sSearch)) {
            $where = "uacc_id = '" . $this->flexi_auth->get_user_id() . "' AND (";
            $or = false;
            for ($i = 0; $i < count($aColumns); $i++) {
                $bSearchable = $this->input->get_post('bSearchable_' . $i, true);

                // Individual column filtering
                if (isset($bSearchable) && $bSearchable == 'true') {
                    if (!$or) {
                        $where .= "`" . $aColumns[$i] . "` LIKE '%" . $this->db->escape_like_str($sSearch) . "%'";
                    } else {
                        $where .= " OR `" . $aColumns[$i] . "` LIKE '%" . $this->db->escape_like_str($sSearch) . "%'";
                    }
                    $or = true;
//                    $this->db->or_like($aColumns[$i], $this->db->escape_like_str($sSearch));
                }
            }
            $where .= ") ";
            $this->db->where($where);
        } else {
            $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
        }

//        $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
        $this->db->join('ticket_types', 'ticket_types.type_id = ticket_list.type_id');

        // Select Data
        $aColumnsTmp = array('ticket_id', 'name', 'company', 'active', '\'1\' AS winning', 'added', 'date_from');
        $this->db->select('SQL_CALC_FOUND_ROWS ' . str_replace(' , ', ' ', implode(', ', $aColumnsTmp)), false);
        $rResult = $this->db->get($sTable);
        $tmp = $this->db->last_query();

        // Data set length after filtering
        $this->db->select('FOUND_ROWS() AS found_rows');
        $iFilteredTotal = $this->db->get()->row()->found_rows;

        // Total data set length
        $this->db->select('uacc_id');
        $this->db->from('ticket_list');
        $this->db->where('uacc_id', $this->flexi_auth->get_user_id());
        $iTotal = $this->db->count_all_results();

        // Output
        $output = array(
            'sEcho' => intval($sEcho),
            'iTotalRecords' => $iTotal,
            'iTotalDisplayRecords' => $iFilteredTotal,
            'aaData' => array()
        );

        $this->load->library('result_verification');

        foreach ($rResult->result_array() as $aRow) {
            $row = array();

            foreach ($aColumns as $col) {
                if ($col == 'added') {
                    $tmp = strtotime($aRow[$col]);

                    if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
                        $row[] = iconv('', 'utf-8', strftime("%a, %#d. %B %Y %H:%M", $tmp));
                    } else {
                        $row[] = iconv('', 'utf-8', strftime("%a, %e. %B %Y %H:%M", $tmp));
                    }

                } else if ($col == 'date_from') {
                    $tmp = strtotime($aRow[$col]);

                    if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN') {
                        $row[] = iconv('', 'utf-8', strftime("%a, %#d. %B %Y", $tmp));
                    } else {
                        $row[] = iconv('', 'utf-8', strftime("%a, %e. %B %Y", $tmp));
                    }
                } else if ($col == 'winning') {
//                    $row[] = $aRow[$col];
                    $row[] = $this->result_verification->check_ticket_result($aRow['ticket_id']);
                } else {
                    $row[] = $aRow[$col];
                }
            }

            $row[] = anchor(lang('link_ticket') . '/' . lang('link_view') . '/' . $aRow['ticket_id'], '<img src=' . asset_img() . 'view.png >', 'title="Zobraziť tiket"');
            $row[] = anchor(lang('link_ticket') . '/' . lang('link_edit') . '/' . $aRow['ticket_id'], '<img src=' . asset_img() . 'edit.png >', 'title="Upraviť tiket"');
            $row[] = anchor(lang('link_ticket') . '/' . lang('link_delete_ajax') . '/' . $aRow['ticket_id'], '<img src=' . asset_img() . 'delete.png >', array('title' => 'Odstrániť tiket', 'class' => 'deleteicon'));

            $output['aaData'][] = $row;
        }

        //custom sort winning column
        if (isset($iSortCol_0)) {
            for ($i = 0; $i < intval($iSortingCols); $i++) {
                $iSortCol = $this->input->get_post('iSortCol_' . $i, true);
                $bSortable = $this->input->get_post('bSortable_' . intval($iSortCol), true);
                $sSortDir = $this->input->get_post('sSortDir_' . $i, true);

                if ($bSortable == 'true') {
                    if ($aColumns[$iSortCol] == 'winning') {
                        if ($sSortDir == 'asc') {
                            usort($output['aaData'], function ($a, $b) {
                                return $b[4] - $a[4];
                            });
                        } else {
                            usort($output['aaData'], function ($a, $b) {
                                return $a[4] - $b[4];
                            });
                        }
                    }
                }
            }
        }

        echo json_encode($output);
    }

}